@@ -6,7 +6,7 @@
 #ifdef POPCNT_INTRINSICS
 static __inline__ mp_bitcnt_t shortCount(slong val)
 {
-#if defined(_WIN64)
+#if defined(_WIN64) || defined(__mips64)
    return __builtin_popcountll(val);  
 #else
    return __builtin_popcountl(val);
