@@ -20,7 +20,7 @@
 /******************************************************************************
 
     Copyright (C) 2009 William Hart
-    Copyright (C) 2014 Dana Jacobsen
+    Copyright (C) 2014, 2015 Dana Jacobsen
 
 ******************************************************************************/
 
@@ -46,6 +46,11 @@ int n_is_prime(mp_limb_t n)
         !(n%23) || !(n%29) || !(n%31) || !(n%37) ||
         !(n%41) || !(n%43) || !(n%47) || !(n%53))   return 0;
     if (n < 3481) /* 59*59 */                       return 1;
+    if (n > 1000000 &&
+        (!(n% 59) || !(n% 61) || !(n% 67) || !(n% 71) || !(n% 73) ||
+         !(n% 79) || !(n% 83) || !(n% 89) || !(n% 97) || !(n%101) ||
+         !(n%103) || !(n%107) || !(n%109) || !(n%113) || !(n%127) ||
+         !(n%131) || !(n%137) || !(n%139) || !(n%149)))  return 0;
 
     return n_is_probabprime(n);
 }
